@@ -25,7 +25,7 @@ SOFTWARE.
 import XDHqSHRD
 from XDHqSHRD import getEnv
 
-import inspect, os, socket, sys, threading
+import inspect, os, socket, sys, threading, time
 
 if sys.version_info[0] == 2:
 	import XDHqFaaS2
@@ -34,6 +34,7 @@ if sys.version_info[0] == 2:
 	_writeString = XDHqFaaS2.writeString
 	_readUInt = XDHqFaaS2.readUInt
 	_getString = XDHqFaaS2.getString
+	_exitThread = XDHqFaaS2.exitThread
 else:
 	import XDHqFaaS3
 	l = XDHqFaaS3.l
@@ -41,6 +42,7 @@ else:
 	_writeString = XDHqFaaS3.writeString
 	_readUInt = XDHqFaaS3.readUInt
 	_getString = XDHqFaaS3.getString
+	_exitThread = XDHqFaaS3.exitThread
 
 _bye = False	# For use in Jupiter notbooks, to quit an application.
 
@@ -74,46 +76,61 @@ _MAIN_PROTOCOL_LABEL = "bf077e9f-baca-48a1-bd3f-bf5181a78666"
 _MAIN_PROTOCOL_VERSION = "0"
 
 _writeLock = threading.Lock()
-_globalCondition = threading.Condition()
+_readLock = threading.Event()
+
+
+def waitForInstance():
+	_readLock.wait()
+	_readLock.clear()
+
+
+def instanceDataRead():
+	_readLock.set()
+
 
 _url = ""
 
 class Instance:
 	def __init__(self):
-		self.condVar = threading.Condition()
+		# https://github.com/epeios-q37/atlas-python/pull/7 (Condition -> Event)
+		self._readLock = threading.Event()
 		self.handshakeDone = False
-		self.quit = False;
+		self.quit = False
+	def __del__(self):
+		_report("Inst.  #" + str(self.id) + " closed!")
 	def set(self,thread,id):
 		self.thread = thread
 		self.id = id
 	def IsHandshakeDone(self):
 		if self.handshakeDone:
 			return True
-		else:
-			self.handshakeDone = True
-			return False
+
+		self.handshakeDone = True
+		return False
 	def getId(self):
 		return self.id
-	def wait(self):
-		with self.condVar:
-			self.condVar.wait()
-	def signal(self):
-		with self.condVar:
-			self.condVar.notify()
+	def waitForData(self):
+		self._readLock.wait()
+		self._readLock.clear()
+		if self.quit:
+			instanceDataRead()
+			_exitThread()
+	def dataAvailable(self):
+		self._readLock.set()
 
 def isTokenEmpty():
 	return not _token or _token[0] == "&"
 
 def writeUInt(value):
 	global _socket
-	_writeUInt( _socket, value )
+	_writeUInt( _socket, value, lambda: _bye )
 
 def writeSInt(value):
 	writeUInt( ( ( -value - 1 ) << 1 ) | 1 if value < 0 else value << 1 )
 
 def writeString(string):
 	global _socket
-	_writeString(_socket, string)
+	_writeString(_socket, string, lambda: _bye)
 
 def writeStrings(strings):
 	writeUInt(len(strings))
@@ -143,6 +160,17 @@ def getStrings():
 
 	return strings
 
+def _dismiss(id):
+	with _writeLock:
+		writeSInt(id)
+		writeString("#Dismiss_1")
+
+def _report(message):
+	with _writeLock:
+		writeSInt(-1)
+		writeString("#Inform_1")
+		writeString(message)
+
 def _init():
 	global _token, _socket, _wAddr, _wPort, _cgi
 	pAddr = "faas1.q37.info"
@@ -183,14 +211,9 @@ def _init():
 	_socket.settimeout(1)	# In order to quit an application, in Jupyter notebooks.		
 
 def _handshake():
-	global _writeLock
-
-	_writeLock.acquire()
-
-	writeString(_FAAS_PROTOCOL_LABEL)
-	writeString(_FAAS_PROTOCOL_VERSION)
-
-	_writeLock.release()
+	with _writeLock:
+		writeString(_FAAS_PROTOCOL_LABEL)
+		writeString(_FAAS_PROTOCOL_VERSION)
 
 	error = getString()
 
@@ -204,14 +227,11 @@ def _handshake():
 
 def _ignition():
 	global _token, _url
-	_writeLock.acquire()
-
-	writeString( _token)
-	writeString(_headContent)
-	writeString(_wAddr);
-	writeString("PYH")
-
-	_writeLock.release()
+	with _writeLock:
+		writeString( _token)
+		writeString(_headContent)
+		writeString(_wAddr);
+		writeString("PYH")
 
 	_token = getString()
 
@@ -228,7 +248,6 @@ def _ignition():
 		_supply(_url)
 
 def _serve(callback,userCallback,callbacks ):
-	global _writeLock, _globalCondition
 	while True:
 		id = readSInt()
 		
@@ -238,28 +257,32 @@ def _serve(callback,userCallback,callbacks ):
 			id = readSInt()  # The id of the new session.
 
 			if id in _instances:
-				sys.exit("Instance of id '" + str(id) + "' exists but should not !")
-
+				_report("Instance of id '" + str(id) + "' exists but should not !")
 			instance = Instance()
 			instance.set(callback(userCallback, callbacks, instance),id)
 			_instances[id] = instance
 
-			_writeLock.acquire()
-			writeSInt(id)
-			writeString(_MAIN_PROTOCOL_LABEL)
-			writeString(_MAIN_PROTOCOL_VERSION)
-			_writeLock.release()
+			with _writeLock:
+				writeSInt(id)
+				writeString(_MAIN_PROTOCOL_LABEL)
+				writeString(_MAIN_PROTOCOL_VERSION)
 		elif id == -3:	# Value instructing that a session is closed.
 			id = readSInt();
+
 			if not id in _instances:
-				sys.exit("Instance of id '" + str(id) + "' not available for destruction!")
-			_instances[id].quit = True
-			_instances[id].signal();
-			with _globalCondition:
-				_globalCondition.wait()
-			del _instances[id]	# Seemingly destroy the object and remove the entry too.
+				_report("Instance of id '" + str(id) + "' not available for destruction!")
+			else:
+				instance = _instances.pop(id)
+				instance.quit = True
+				instance.dataAvailable()
+				waitForInstance()
+				instance = None # Without this, instance will only be destroyed
+												# when 'instance" is set to a new instance.
 		elif not id in _instances:
-			sys.exit("Unknown instance of id '" + str(id) + "'!")
+			message = "Unknown instance of id '" + str(id) + "'!"
+			print(message)
+			_report(message)
+			_dismiss(id)
 		elif not _instances[id].IsHandshakeDone():
 			error = getString()
 
@@ -268,15 +291,14 @@ def _serve(callback,userCallback,callbacks ):
 
 			getString()	# Language. Not handled yet.
 		else:
-			_instances[id].signal()
+			_instances[id].dataAvailable()
+			waitForInstance()
 
-			with _globalCondition:
-				_globalCondition.wait()
 
-def launch(callback, userCallback,callbacks,headContent):
+def launch(callback, userCallback, callbacks, headContent):
 	global _headContent, _instances
 
-	if headContent == None:
+	if headContent is None:
 		if not "_headContent" in globals():
 			_headContent = "" 
 	else:
@@ -295,12 +317,11 @@ def launch(callback, userCallback,callbacks,headContent):
 def get_app_url(id=""):
 	return _url + ("&_id=" + str(id) if id else "") 
 
-def broadcastAction(action,id=""):
-	_writeLock.acquire()
-	writeSInt(-3)
-	writeString(action)
-	writeString(id)
-	_writeLock.release()	
+def broadcastAction(action, id = ""):
+	with _writeLock:
+		writeSInt(-3)
+		writeString(action)
+		writeString(id)
 
 class DOM_FaaS:
 	_firstLaunch = True
@@ -308,71 +329,62 @@ class DOM_FaaS:
 	def __init__(self, instance):
 		self.instance = instance
 
-	def wait(self):
-		self.instance.wait()
+	def waitForData(self):
+		self.instance.waitForData()
 		
-	def signal(self):
-		with _globalCondition:
-			_globalCondition.notify()
-
-	def _sendSpecialAction(self, action):
-		_writeLock.acquire()
-		writeSInt(self.instance.getId())
-		writeString(action)
-		_writeLock.release()
+	def _standBy(self):
+		with _writeLock:
+			writeSInt(self.instance.getId())
+			writeString("#StandBy_1")
 
 	def getAction(self):
 		if self._firstLaunch:
 			self._firstLaunch = False
 		else:
-			self._sendSpecialAction("#StandBy_1")
+			self._standBy()
 
-		self.wait()
+		self.waitForData()
 
-		[id,action]=["",""] if self.instance.quit else [getString(),getString()]
+		[id,action] = [getString(),getString()]
 
-		# The below 'is_quitting()' method MUST be called, or the library will hang. 
+		instanceDataRead()
 
 		return [action,id]
 
-	def isQuitting(self):
-		answer = self.instance.quit
-
-		# Below line were in 'getAction()', but, in case of quitting,
-		# 'self.instance' could already be destroyed here.
-		self.signal()
-
-		return answer;
-
-
 	def call(self, command, type, *args):
-		_writeLock.acquire()
-		writeSInt(self.instance.getId())
-		writeString(command)
+		if self.instance.quit:
+			"""
+			Below function unlocks the main thread,
+			and exits the thread corresponding
+			to the current instance.
+			"""
+			self.waitForData()
 
-		writeUInt(type)
+		with _writeLock:
+			writeSInt(self.instance.getId())
+			writeString(command)
 
-		for arg in args:
-			if isinstance(arg,str):
-				writeUInt(XDHqSHRD.RT_STRING)
-				writeString(arg)
-			else:
-				writeUInt(XDHqSHRD.RT_STRINGS)
-				writeStrings(arg)
+			writeUInt(type)
 
-		writeUInt(XDHqSHRD.RT_VOID)	# To report end of argument list.
+			for arg in args:
+				if isinstance(arg,str):
+					writeUInt(XDHqSHRD.RT_STRING)
+					writeString(arg)
+				else:
+					writeUInt(XDHqSHRD.RT_STRINGS)
+					writeStrings(arg)
 
-		_writeLock.release()
+			writeUInt(XDHqSHRD.RT_VOID)	# To report end of argument list.
 
 		if type == XDHqSHRD.RT_STRING:
-			self.wait()
+			self.waitForData()
 			string = getString()
-			self.signal()
+			instanceDataRead()
 			return string
 		elif type == XDHqSHRD.RT_STRINGS:
-			self.wait()
+			self.waitForData()
 			strings = getStrings()
-			self.signal()
+			instanceDataRead()
 			return strings
 		elif type != XDHqSHRD.RT_VOID:
 			sys.exit("Unknown return type !!!")
